@@ -1,18 +1,20 @@
 from PyQt5.QtGui import QIcon, QPixmap
-from PyQt5.QtWidgets import QMainWindow
+from PyQt5.QtWidgets import QApplication, QMainWindow
 
 
 class WindowController:
     """ Wrapper class with methods for manipulating the main window. """
 
     def __init__(self, w_window:QMainWindow) -> None:
-        self._w_window = w_window   # Main Qt window. All dialogs must be children of this widget.
+        self._w_window = w_window  # Main Qt window.
 
     def show(self) -> None:
-        """ Show the window, move it in front of other windows, and activate focus. """
+        """ Show the window, move it in front of other windows, and activate focus.
+            These events should be explicitly processed before another thread can take the GIL. """
         self._w_window.show()
         self._w_window.activateWindow()
         self._w_window.raise_()
+        QApplication.instance().processEvents()
 
     def close(self) -> None:
         self._w_window.close()
